# Don't process requires for .py* files, since it massively increases build time
# and we already know that we depend Python -- we're building it!
%define __requires_exclude_from ^%{_libdir}/.*[.]py[oc]?$

%define major 3
%define minor 9
%define fix 18
%define priority %(printf '%02d%02d' %{major} %{minor})

Name: python39
Version: %{major}.%{minor}.%{fix}
Release: 1
License: Python-2.0
Summary: Python 3 Programming Language
Url: https://www.python.org

BuildRequires: autoconf
BuildRequires: automake
BuildRequires: autoconf-archive
BuildRequires: bzip2-devel
BuildRequires: expat-devel
BuildRequires: libffi-devel
BuildRequires: libtool
BuildRequires: libutil-devel >= 0.7
BuildRequires: libuuid-devel
BuildRequires: ncurses-devel
BuildRequires: openssl-1.1-devel
BuildRequires: pkg-config
BuildRequires: readline-devel
BuildRequires: sqlite3-devel
BuildRequires: tk-devel
BuildRequires: xz-devel
BuildRequires: zlib-devel

# if this is available, _locale will link to it
# Is there a benefit?
# BuildRequires: libintl-devel

Requires: libutil2 >= 0.7
Requires(post): update-alternatives
Requires(postun): update-alternatives

Source0: https://www.python.org/ftp/python/%{version}/Python-%{version}.tar.xz

Patch0: python39-_large_files.patch
Patch1: python39-dev_t-64bit.patch
Patch2: python39-_posixshmem.patch
Patch3: python39-flock-lockf-eacces.patch
Patch4: python39-makexp_aix.patch
Patch5: python39-ibmi-support.patch
Patch6: python39-named-semaphore.patch
Patch7: python39-struct-pwd.patch
Patch8: python39-thread_cputime.patch
Patch9: python39-itimer_prof.patch
Patch10: python39-o_nofollow.patch
Patch11: python39-o_cloexec.patch
Patch12: python39-o_search.patch
Patch13: python39-clock_getres.patch
Patch14: python39-clock-cputime-id.patch
Patch15: python39-posix_spawnattr_init.patch
Patch16: python39-install-aix-build-utils.patch
Patch17: python39-ctypes-alignment.patch
# This patch should not be brought forward to future Python 3 versions
Patch18: python39-sys-platform-compat.patch

%description
Python is a programming language that lets you work more quickly and integrate
your systems more effectively.

You can learn to use Python and see almost immediate gains in productivity and
lower maintenance costs.

%package devel
Summary: Python 3 development files
Requires: %{name} = %{version}-%{release}
# TODO: The following is supported by rpm, but not by yum
# it ensures that python-rpm-macros is only required when building rpms
# Requires (python-rpm-macros if rpm-build)
Requires: python-rpm-macros

%description devel
Python 3 development files

%package tkinter
Summary: A graphical user interface for the Python 3 language

%description tkinter
The Tkinter (Tk interface) library is a graphical user interface toolkit for
the Python programming language.

%prep
%setup -q -n Python-%{version}

%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1
%patch10 -p1
%patch11 -p1
%patch12 -p1
%if 0%{?ibmi_version} <= 73
%patch13 -p1
%endif
%patch14 -p1
%patch15 -p1
%patch16 -p1
%patch17 -p1
%patch18 -p1

%build

autoreconf -fiv

%configure \
  CC='gcc' \
  CXX=g++ \
  CPPFLAGS="-I%{_includedir} -I%{_includedir}/ncursesw -I%{_includedir}/uuid -D_ALL_SOURCE -D_XOPEN_SOURCE=700" \
  LDFLAGS="-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib" \
  LDFLAGS_NODIST="-lutil" \
  ARFLAGS="-X${OBJECT_MODE} rc" \
  CCSHARED="-fPIC" \
  ac_cv_flock_decl=no \
  ac_cv_func_flock=no \
  ac_cv_lib_bsd_flock=no \
  ac_cv_func_clock_settime=no \
  ac_cv_func_posix_fadvise=no \
  ac_cv_func_posix_fallocate=no \
  ac_cv_func_sched_get_priority_max=no \
  ac_cv_func_sched_rr_get_interval=no \
  ac_cv_func_sched_setaffinity=no \
  ac_cv_func_sched_setparam=no \
  ac_cv_func_sched_setscheduler=no \
  ac_cv_func_fexecve=no \
  ac_cv_func_faccessat=no \
  ac_cv_func_fchmodat=no \
  ac_cv_func_fchownat=no \
  ac_cv_func_fstatat=no \
  ac_cv_func_futimesat=no \
  ac_cv_func_linkat=no \
  ac_cv_func_lstat=yes \
  ac_cv_func_mkdirat=no \
  ac_cv_func_mkfifoat=no \
  ac_cv_func_mknodat=no \
  ac_cv_func_openat=no \
  ac_cv_func_readlinkat=no \
  ac_cv_func_renameat=no \
  ac_cv_func_symlinkat=no \
  ac_cv_func_unlinkat=no \
  ac_cv_func_utimensat=no \
  ac_cv_func_shm_open=no \
  ac_cv_func_shm_unlink=no \
  ac_cv_func_setregid=no \
  ac_cv_enable_visibility=no \
  --with-system-expat \
  --with-system-ffi \
  --with-ensurepip=no \
  --with-tcltk-includes="$(pkg-config --cflags tk)" \
  --with-tcltk-libs="$(pkg-config --libs tk)" \
  --with-computed-gotos \
  --with-tzpath=/QOpenSys/usr/share/lib/zoneinfo \
  --enable-ipv6 \
  --enable-loadable-sqlite-extensions \
  --enable-shared \
  #end

perl -p -i -e 's|#! /usr/local/bin/python|#! %{_bindir}/python3|' Lib/cgi.py

%make_build python

# Hack to re-build python binary without -brtl
gcc -pthread "-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib" \
    -lutil  -Wl,-bE:Modules/python.exp -lld -o python \
    Programs/python.o libpython3.9.so -ldl  -lutil -lm   -lm \
    -Wl,-bnortl -Wl,-blibsuff:so

# build extensions with single make job due to thread executer issues
make

%install

%make_install

rm %{buildroot}/%{_bindir}/python3
rm %{buildroot}/%{_bindir}/2to3
rm %{buildroot}/%{_bindir}/idle3
rm %{buildroot}/%{_bindir}/pydoc3
rm %{buildroot}%{_bindir}/python%{major}-config
rm %{buildroot}%{_mandir}/man1/python3.1
rm %{buildroot}%{_libdir}/pkgconfig/python3-embed.pc
rm %{buildroot}%{_libdir}/pkgconfig/python3.pc
rm %{buildroot}%{_libdir}/python%{major}.%{minor}/LICENSE.txt

mv python-gdb.py %{buildroot}%{_libdir}/python%{major}.%{minor}

%post
%{_bindir}/update-alternatives \
  --install %{_bindir}/python%{major} python%{major} %{_bindir}/python%{major}.%{minor} %{priority} \
  --slave %{_bindir}/2to3 2to3 %{_bindir}/2to3-%{major}.%{minor} \
  --slave %{_bindir}/idle3 idle3 %{_bindir}/idle%{major}.%{minor} \
  --slave %{_bindir}/pydoc3 pydoc3 %{_bindir}/pydoc%{major}.%{minor} \

%{_bindir}/update-alternatives --install %{_bindir}/python python %{_bindir}/python%{major}.%{minor} %{priority}

%postun
%{_bindir}/update-alternatives --remove python%{major} %{_bindir}/python%{major}
%{_bindir}/update-alternatives --remove python %{_bindir}/python

%files
%defattr(-, qsys, *none)

%license LICENSE

%{_bindir}/2to3-%{major}.%{minor}
%{_bindir}/idle%{major}.%{minor}
%{_bindir}/pydoc%{major}.%{minor}
%{_bindir}/python%{major}.%{minor}

%ghost %{_bindir}/python3
%ghost %{_bindir}/2to3
%ghost %{_bindir}/idle3
%ghost %{_bindir}/pydoc3
%ghost %{_bindir}/python

%{_libdir}/libpython%{major}.%{minor}.so

%dir %{_libdir}/python%{major}.%{minor}/

%{_libdir}/python%{major}.%{minor}/*.py
%exclude %{_libdir}/python%{major}.%{minor}/turtle.py
%exclude %{_libdir}/python%{major}.%{minor}/python-gdb.py

%{_libdir}/python%{major}.%{minor}/__pycache__/
%{_libdir}/python%{major}.%{minor}/asyncio/
%{_libdir}/python%{major}.%{minor}/collections/
%{_libdir}/python%{major}.%{minor}/concurrent/
%{_libdir}/python%{major}.%{minor}/ctypes/
%{_libdir}/python%{major}.%{minor}/curses/
%{_libdir}/python%{major}.%{minor}/dbm/
%{_libdir}/python%{major}.%{minor}/distutils/
%{_libdir}/python%{major}.%{minor}/email/
%{_libdir}/python%{major}.%{minor}/encodings/
%{_libdir}/python%{major}.%{minor}/ensurepip/
%{_libdir}/python%{major}.%{minor}/html/
%{_libdir}/python%{major}.%{minor}/http/
%{_libdir}/python%{major}.%{minor}/idlelib/
%{_libdir}/python%{major}.%{minor}/importlib/
%{_libdir}/python%{major}.%{minor}/json/
%{_libdir}/python%{major}.%{minor}/lib2to3/
%{_libdir}/python%{major}.%{minor}/logging/
%{_libdir}/python%{major}.%{minor}/multiprocessing/
%{_libdir}/python%{major}.%{minor}/pydoc_data/
%{_libdir}/python%{major}.%{minor}/site-packages/
%{_libdir}/python%{major}.%{minor}/sqlite3/
%{_libdir}/python%{major}.%{minor}/test/
%{_libdir}/python%{major}.%{minor}/unittest/
%{_libdir}/python%{major}.%{minor}/urllib/
%{_libdir}/python%{major}.%{minor}/venv/
%{_libdir}/python%{major}.%{minor}/wsgiref/
%{_libdir}/python%{major}.%{minor}/xml/
%{_libdir}/python%{major}.%{minor}/xmlrpc/
%{_libdir}/python%{major}.%{minor}/zoneinfo/

%dir %{_libdir}/python%{major}.%{minor}/lib-dynload/
%{_libdir}/python%{major}.%{minor}/lib-dynload/_asyncio.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_bisect.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_blake2.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_bz2.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_cn.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_hk.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_iso2022.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_jp.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_kr.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_tw.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_contextvars.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_crypt.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_csv.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ctypes.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ctypes_test.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_curses.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_curses_panel.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_datetime.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_dbm.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_decimal.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_elementtree.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_hashlib.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_heapq.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_json.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_lsprof.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_lzma.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_md5.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_multibytecodec.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_multiprocessing.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_opcode.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_pickle.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_posixsubprocess.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_queue.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_random.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha1.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha256.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha3.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha512.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_socket.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sqlite3.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_statistics.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ssl.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_struct.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testbuffer.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testcapi.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testimportmultiple.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testinternalcapi.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testmultiphase.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_uuid.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_xxsubinterpreters.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_xxtestfuzz.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_zoneinfo.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/array.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/audioop.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/binascii.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/cmath.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/fcntl.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/grp.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/math.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/mmap.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/nis.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/parser.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/pyexpat.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/readline.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/resource.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/select.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/syslog.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/termios.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/unicodedata.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/xxlimited.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/zlib.cpython-%{major}%{minor}.so

%{_mandir}/man1/*

%files devel
%defattr(-, qsys, *none)
%{_bindir}/python%{major}.%{minor}-config

%{_includedir}/python%{major}.%{minor}

%dir %{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/Makefile
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/Setup
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/Setup.local
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/config.c
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/config.c.in
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/install-sh
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/ld_so_aix
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/libpython%{major}.%{minor}.a
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/makesetup
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/makexp_aix
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/python-config.py
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/python.exp
%{_libdir}/python%{major}.%{minor}/config-%{major}.%{minor}/python.o

%{_libdir}/python%{major}.%{minor}/python-gdb.py

%{_libdir}/pkgconfig/python-3.9-embed.pc
%{_libdir}/pkgconfig/python-3.9.pc

%files tkinter
%defattr(-, qsys, *none)
%{_libdir}/python%{major}.%{minor}/lib-dynload/_tkinter.cpython-%{major}%{minor}.so
%{_libdir}/python%{major}.%{minor}/tkinter
%{_libdir}/python%{major}.%{minor}/turtle.py
%{_libdir}/python%{major}.%{minor}/turtledemo


%changelog
* Thu Nov 02 2023 Kevin Adler <kadler@us.ibm.com> - 3.9.18
- Build python3 binary with -bnortl to allow packages to load multiple OpenSSL
  without conflicts
- Update to 3.9.18

* Thu Feb 09 2023 Kevin Adler <kadler@us.ibm.com> - 3.9.16
- BuildRequire openssl-1.1-devel instead of openssl-devel

* Wed Feb 01 2023 Xu Meng <mengxumx@cn.ibm.com> - 3.9.16
- Update to 3.9.16

* Mon Sep 19 2022 Xu Meng <mengxumx@cn.ibm.com> - 3.9.14
- Update to 3.9.14

* Thu Mar 24 2022 Kevin Adler <kadler@us.ibm.com> - 3.9.11
- Update to 3.9.11

* Wed Mar 02 2022 Kevin Adler <kadler@us.ibm.com> - 3.9.10
- Alter sys.platform to return "aix7" instead of "os400" to match Python 3.6
  behavior when environment variable PYTHON_PLATFORM_COMPAT_IBMI is set to "Y"

* Thu Feb 03 2022 Kevin Adler <kadler@us.ibm.com> - 3.9.10
- Update to 3.9.10
- Patch ctypes to ensure 16-byte alignment for small types

* Thu Dec 16 2021 Abdirahim Musse <abdirahim.musse@ibm.com>
- Update to 3.9.8

* Wed Dec 15 2021 Abdirahim Musse <abdirahim.musse@ibm.com>
- Update makexp_aix in Python to not demangle C++ symbols

* Thu Nov 4 2021 Jason Hansen <stymie@us.ibm.com>
- Intra-package Requires need version and release

* Wed Jul 07 2021 Kevin Adler <kadler@us.ibm.com>
- Update to 3.9.6

* Wed Jul 07 2021 Abdirahim Musse <abdirahim.musse@ibm.com>
- Configure correct tzpath

* Fri Jun 18 2021 Kevin Adler <kadler@us.ibm.com>
- Require python-rpm-macros in devel package

* Mon Jun 14 2021 Kevin Adler <kadler@us.ibm.com>
- Add patch to install AIX build utils on IBM i as well
- Explicitly list python files to prevent bad builds
- Use version macros in dynload .so files

* Tue Jun 8 2021 Abdirahim Musse<abdirahim.musse@ibm.com>
- Initial spec
